"""
AvaTax Software Development Kit for Python.

   Copyright 2019 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

@author     Robert Bronson
@author     Phil Werner
@author     Adrienne Karnoski
@author     Han Bao
@copyright  2019 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    TBD
@link       https://github.com/avadev/AvaTax-REST-V2-Python-SDK
"""
from requests.auth import HTTPBasicAuth
from ._str_version import str_type
from . import client_methods
import os


class AvataxClient(client_methods.Mixin):
    """Class for our Avatax client."""

    def __init__(self, app_name=None, app_version=None, machine_name=None,
                 environment=None, timeout_limit=None):
        """
        Initialize the sandbox client.

        By default the client object environment will be production. For
        sandbox API, set the envirnoment variable to sandbox.

            :param  string  app_name: The name of your Application
            :param  string/integer  app_version:  Version of your Application
            :param  string  machine_name: Name of machine you are working on
            :param  string  environment: Default environment is production,
                input sandbox, for the sandbox API
            :param  int/float The timeout limit for every call made by this client instance. (default: 10 sec)
        :return: object
        """
        if not all(isinstance(i, str_type) for i in [app_name,
                                                     machine_name,
                                                     environment]):
            raise ValueError('Input(s) must be string or none type object')
        self.base_url = 'https://rest.avatax.com'
        if environment:
            if environment.lower() == 'sandbox':
                self.base_url = 'https://sandbox-rest.avatax.com'
            elif environment[:8] == 'https://' or environment[:7] == 'http://':
                self.base_url = environment
        self.auth = None
        self.app_name = app_name
        self.app_version = app_version
        self.machine_name = machine_name
        self.client_id = '{}; {}; Python SDK; API_VERSION; {};'.format(app_name,
                                                                app_version,
                                                                machine_name)
        self.client_header = {'X-Avalara-Client': self.client_id}
        self.timeout_limit = timeout_limit 

    def add_credentials(self, username=None, password=None):
        """
        Configure this client for the specified username/password security.

        :param  string  username:    The username of your AvaTax user account
        :param  string  password:    The password of your AvaTax user account
        :param  int     accountId:   The account ID of your avatax account
        :param  string  licenseKey:  The license key of your avatax account
        :param  string  bearerToken: The OAuth 2.0 token provided by Avalara
        :return: AvaTaxClient

        Note: if you wish to use Bearer token, enter it as the ONLY argument to this method.
        """
        if not all(isinstance(i, str_type) for i in [username, password]):
            raise ValueError('Input(s) must be string or none type object')
        if username and not password:
            self.client_header['Authorization'] = 'Bearer ' + username
        else:
            self.auth = HTTPBasicAuth(username, password)
        return self

# to generate a client object on initialization of this file, uncomment the script below
# if __name__ == '__main__':  # pragma no cover
#     """Creating a client with credential, must have env variables username & password."""
#     client = AvataxClient('my test app',
#                           'ver 0.0',
#                           'my test machine',
#                           'sandbox')
#     c = client.add_credentials(os.environ.get('USERNAME', ''),
#                                os.environ.get('PASSWORD', ''))
#     print(client.ping().text)
#     tax_document = {
#         'addresses': {'SingleLocation': {'city': 'Irvine',
#                                          'country': 'US',
#                                          'line1': '123 Main Street',
#                                          'postalCode': '92615',
#                                          'region': 'CA'}},
#         'commit': False,
#         'companyCode': 'DEFAULT',
#         'currencyCode': 'USD',
#         'customerCode': 'ABC',
#         'date': '2017-04-12',
#         'description': 'Yarn',
#         'lines': [{'amount': 100,
#                   'description': 'Yarn',
#                    'itemCode': 'Y0001',
#                    'number': '1',
#                    'quantity': 1,
#                    'taxCode': 'PS081282'}],
#         'purchaseOrderNo': '2017-04-12-001',
#         'type': 'SalesInvoice'}



"""Set up file, to install this module : pip install -e ."""
from setuptools import setup


pkg_description = ''
with open('README.md') as f:
    pkg_description = f.read()


setup(
    name='Avalara',
    version='22.7.0',
    url='https://github.com/avadev/AvaTax-REST-V2-Python-SDK',
    package_dir={'': 'src'},
    packages=['avalara'],
    author='Han Bao, Adrienne Karnoski, Robert Bronson, Philip Werner, Genevieve Conty',
    author_email='han.bao@avalara.com',
    description='Avalara Tax Python SDK.',
    long_description=pkg_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6'
    ],
    install_requires=['requests'],
    extras_require={
        "test": ['pytest', 'pytest-cov', 'tox']
    })



"""
AvaTax Software Development Kit for Python.

   Copyright 2019 Avalara, Inc.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

@author     Robert Bronson
@author     Phil Werner
@author     Adrienne Karnoski
@author     Han Bao
@copyright  2019 Avalara, Inc.
@license    https://www.apache.org/licenses/LICENSE-2.0
@version    TBD
@link       https://github.com/avadev/AvaTax-REST-V2-Python-SDK

Transaction Builder methods(to be auto generated)
"""


class Mixin:
    """Mixin containing methods attached to TransactionBuilder Class."""

    def with_commit(self):
        """
        Set the commit flag of the transaction.

        If commit is set to False, the transaction will only be saved.
        """
        self.create_model['commit'] = True
        return self

    def with_transaction_code(self, code):
        """
        Set a specific transaction code.

        :param  string  code:  Specific tansaction code
        :return:  TransactionBuilder
        """
        self.create_model['code'] = code
        return self

    def with_type(self, type_):
        r"""
        Set the document type.

        :param  string  type: Address Type \
        (See DocumentType::* for a list of allowable values)

        :return: TransactionBuilder
        """
        self.create_model['type'] = type_
        return self

    def with_address(self, address_type, address):
        r"""
        Add an address to this transaction.

        :param  string  address_type:  Address Type \
        (See AddressType::* for a list of allowable values)

        :param  dictionary  address:  A dictionary containing the following
            line1         The street address, attention line, or business name
                            of the location.
            line2         The street address, business name, or apartment/unit
                            number of the location.
            line3         The street address or apartment/unit number
                            of the location.
            city          City of the location.
            region        State or Region of the location.
            postal_code    Postal/zip code of the location.
            country       The two-letter country code of the location.
        :return: TransactionBuilder
        """
        self.create_model.setdefault('addresses', {})
        self.create_model['addresses'][address_type] = address
        return self

    def with_line_address(self, address_type, address):
        r"""
        Add an address to this line.

        :param  string  address_type:  Address Type \
        (See AddressType::* for a list of allowable values)
        :param  dictionary  address:  A dictionary containing the following
            line1         The street address, attention line, or business name
                            of the location.
            line2         The street address, business name, or apartment/unit
                            number of the location.
            line3         The street address or apartment/unit number
                            of the location.
            city          City of the location.
            region        State or Region of the location.
            postal_code    Postal/zip code of the location.
            country       The two-letter country code of the location.
        :return: TransactionBuilder
        """
        temp = self.get_most_recent_line('WithLineAddress')
        temp.setdefault('addresses', {})
        temp['addresses'][address_type] = address
        return self

    def with_latlong(self, address_type, lat, long_):
        r"""
        Add a lat/long coordinate to this transaction.

        :param  string  type:       Address Type \
        (See AddressType::* for a list of allowable values)

        :param  float   lat:   The geolocated latitude for this transaction
        :param  float   long_: The geolocated longitude for this transaction

        :return:  TransactionBuilder
        """
        self.create_model.setdefault('addresses', {})
        self.create_model['addresses'][address_type] = {'latitude': float(lat),
                                                        'longitude': float(long_)}
        return self

    def with_line(self, amount, quantity, item_code, tax_code, line_number=None):
        r"""
        Add a line to the transaction.

        :param  float   amount:    Value of the item.
        :param  float   quantity:  Quantity of the item.
        :param  string  item_code: Code of the item.
        :param  string  tax_code:  Tax Code of the item. If left blank, \
        the default item (P0000000) is assumed.
        :param  [int]  line_number: Value of the line number.
        :return:  TransactionBuilder
        """
        if line_number is not None:
            self.line_num = line_number;
        
        temp = {
            'number': str(self.line_num),
            'amount': amount,
            'quantity': quantity,
            'itemCode': str(item_code),
            'taxCode': str(tax_code)
        }
        self.create_model['lines'].append(temp)
        self.line_num += 1
        return self

    def with_exempt_line(self, amount, item_code, exemption_code):
        """
        Add a line with an exemption to this transaction.

        :param   float   amount:         The amount of this line item
        :param   string  item_code:      The code for the item
        :param   string  exemption_code:  The exemption code for this line item
        :return:  TransactionBuilder
        """

        temp = {
            'number': str(self.line_num),
            'quantity': 1,
            'amount': amount,
            'exemptionCode': str(exemption_code),
            'itemCode': str(item_code)
        }
        self.create_model['lines'].append(temp)
        self.line_num += 1
        return self

    def with_diagnostics(self):
        """
        Enable diagnostic information.

        - Sets the debugLevel to 'Diagnostic'
        :return: TransactionBuilder
        """
        self.create_model['debugLevel'] = 'Diagnostic'
        return self

    def with_discount_amount(self, discount):
        """
        Set a specific discount amount.

        :param  float  discount: Amount of the discount
        :return: TransactionBuilder
        """
        self.create_model['discount'] = discount
        return self

    def with_item_discount(self, discounted):
        """
        Set if discount is applicable for the current line.

        :param  boolean  discounted: Set true or false for discounted
        :return: TransactionBuilder
        """
        temp = self.get_most_recent_line('WithItemDiscount')
        temp['discounted'] = discounted
        return self

    def with_parameter(self, name, value):
        """
        Add a parameter at the document level.

        :param  string  name: Name of the parameter
        :param  string  value: Value to be assigned to the parameter
        :return: TransactionBuilder
        """
        self.create_model.setdefault('parameters', {})
        self.create_model['parameters'][name] = value
        return self

    def with_line_parameter(self, name, value):
        """
        Add a parameter to the current line.

        :param  string  name: Name of the parameter
        :param  string  value: Value to be assigned to the parameter
        :return: TransactionBuilder
        """
        temp = self.get_most_recent_line('WithLineParameter')
        temp.setdefault('parameters', {})
        temp['parameters'][name] = value
        return self

    def get_most_recent_line(self, member_name=None):
        """
        Check to see if the current model has a line.

        :return: TransactionBuilder
        """
        line = self.create_model['lines']
        if not len(line):  # if length is zero
            raise Exception('No lines have been added. The {} method applies to the most recent line. To use this function, first add a line.'.format(member_name))
        return line[-1]

    def create(self, include=None):
        """
        Create this transaction.

        :return: TransactionModel
        """
        return self.client.create_transaction(self.create_model, include)

    def with_line_tax_override(self, type_, reason, tax_amount, tax_date):
        r"""
        Add a line-level Tax Override to the current line.

        A TaxDate override requires a valid DateTime object to be passed.

        :param  string  type:        Type of the Tax Override \
        (See TaxOverrideType::* for a list of allowable values)
        :param  string  reason:      Reason of the Tax Override.
        :param  float   tax_amount:  Amount of tax to apply. \
        Required for a TaxAmount Override.
        :param  date    tax_date:    Date of a Tax Override. \
        Required for a TaxDate Override.

        :return:  TransactionBuilder
        """
        line = self.get_most_recent_line('WithLineTaxOverride')
        line['taxOverride'] = {
            'type': str(type_),
            'reason': str(reason),
            'taxAmount': float(tax_amount),
            'taxDate': tax_date
        }
        return self

    def with_tax_override(self, type_, reason, tax_amount, tax_date):
        r"""
        Add a document-level Tax Override to the transaction.

        - A TaxDate override requires a valid DateTime object to be passed

        :param  string  type_:       Type of the Tax Override \
        (See TaxOverrideType::* for a list of allowable values)
        :param  string  reason:      Reason of the Tax Override.
        :param  float   tax_amount:  Amount of tax to apply. Required for \
        a TaxAmount Override.
        :param  date    tax_date:    Date of a Tax Override. Required for \
        a TaxDate Override.
        :return:  TransactionBuilder
        """
        self.create_model['taxOverride'] = {
            'type': str(type_),
            'reason': str(reason),
            'taxAmount': tax_amount,
            'taxDate': tax_date
        }
        return self

    def with_separate_address_line(self, amount, type_, address):
        r"""
        Add a line to this transaction.

        :param  float  amount:  Value of the line
        :param  string  type_:  Address Type  \
        (See AddressType::* for a list of allowable values)
        :param  dictionary  address:  A dictionary containing the following
            line1         The street address, attention line, or business name
                            of the location.
            line2         The street address, business name, or apartment/unit
                            number of the location.
            line3         The street address or apartment/unit number
                            of the location.
            city          City of the location.
            region        State or Region of the location.
            postal_code    Postal/zip code of the location.
            country       The two-letter country code of the location.
        :return: TransactionBuilder
        """
        temp = {
            'number': self.line_num,
            'quantity': 1,
            'amount': amount,
            'addresses': {
                type_: address
            }
        }

        self.create_model['lines'].append(temp)
        self.line_num += 1
        return self

    def create_adjustment_request(self, desc, reason):
        r""".

        Create a transaction adjustment request that can be used with the \
        AdjustTransaction() API call.

        :return: AdjustTransactionModel
        """
        return {
            'newTransaction': self.create_model,
            'adjustmentDescription': desc,
            'adjustmentReason': reason
        }
